## $Id: session_results_presentation.py 16693 2021-11-02 17:21:20Z henrik $
##
## Copyright (C) 2013 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

import grok
from zope.component import getUtility
from zope.interface import implementer, Interface, Attribute
from waeup.kofa.interfaces import IKofaUtils
from waeup.kofa.students.reports.session_results_presentation import (
    SessionResultsPresentation, SessionResultsPresentationGenerator,
    ISessionResultsPresentation, SessionResultsPresentationGeneratorPage,
    SessionResultsPresentationPDFView)

class CustomSessionResultsPresentation(SessionResultsPresentation):

    note = ""

    pdfcreator = ''

    @property
    def introduction (self):
        havenot = ''
        plurals = ''
        num_students = sum([len(i) for i in self.data])
        site = grok.getSite()
        certificate = site[
            'faculties'][self.faccode][self.depcode].certificates[self.certcode]
        study_mode = getattr(certificate, 'study_mode', None)
        if num_students == 0:
            return ''
        if num_students > 1:
            plurals = 's'
            if len(self.data[0]) == num_students:
                havenot = 'have not'
            elif len(self.data[0]):
                havenot = 'have/have not'
            else:
                havenot = 'have'
        else:
            if self.data[0]:
                havenot = 'has not'
            else:
                havenot = 'has'
        if self.faccode == 'FBM':
            board = 'College Academic Board'
        elif study_mode and study_mode.startswith('dp'):
            board = 'Institute Board'
        else:
            board = 'Faculty Board'

        text = """I present to Senate, on behalf of the %s,
the results of courses taught and examinations held this academic session.
The following candidate%s %s satisfied the degree requirements
of the Faculty and %s qualified for the award of a degree as follows:
""" % (board, plurals, havenot, havenot)
        return text

#    note = """
#<br /><br /><br /><br />
#<font size='10'>
#<strong>Note:</strong> This copy is subject to correction for typographical errors and ratification by the departmental board.
#</font>
#"""

    @property
    def signatures(self):
        site = grok.getSite()
        dean = site['faculties'][self.faccode].officer_1
        hod = site['faculties'][self.faccode][self.depcode].officer_1
        examiner = site['faculties'][self.faccode][self.depcode].officer_2
        if not dean:
            dean = 'Dean of Faculty'
        if not hod:
            hod = 'Head of Department'
        if not examiner:
            examiner = 'External Examiner'
        dean = dean.replace(' - ', '<br />')
        hod = hod.replace(' - ', '<br />')
        examiner = examiner.replace(' - ', '<br />')
        return [hod, examiner, dean]

class CustomSessionResultsPresentationGenerator(
    SessionResultsPresentationGenerator):

    def generate(self, site, faccode=None, depcode=None, certcode=None,
                 session=None, level=None, author=None):
        result = CustomSessionResultsPresentation(faccode=faccode,
            depcode=depcode, certcode=certcode, session=session,
            level=level, author=author)
        return result


# Session Results Presentation for graduating students only

class IGradSessionResultsPresentation(ISessionResultsPresentation):
    """Marker interface to distuingish graduating students level reports
    from ordinary level reports.
    """

@implementer(IGradSessionResultsPresentation)
class GradSessionResultsPresentation(CustomSessionResultsPresentation):
    """
    """

    title = 'Presentation of Results to Senate'

    @property
    def right_footer(self):
        return self.title + ' (Graduating Students) - %s -' % self.session

    def _excluded(self, level_obj):
        # Do not list student if selecetd level is not final or an extension
        if level_obj.level < level_obj.student['studycourse'].certificate.end_level:
            return True
        # Filter out students with extension levels higher than selected level
        final_level = int(max(level_obj.student['studycourse'].keys()))
        if final_level > level_obj.level:
            return True
        if level_obj.remark not in (
            'Pass', '3s_rd_s', '2s_2_s', '2s_1_s', '1s_st_s',
            'Merit', 'Credit', 'Distinction'):
            return True
        return False

class GradSessionResultsPresentationGenerator(SessionResultsPresentationGenerator):

    title = 'Presentation of Results to Senate (Graduating Students)'
    grok.name('grad_session_results_presentation')

    def generate(self, site, faccode=None, depcode=None, certcode=None,
                 session=None, level=None, author=None):
        result = GradSessionResultsPresentation(faccode=faccode, depcode=depcode,
            certcode=certcode, session=session, level=level, author=author)
        return result

class GradSessionResultsPresentationGeneratorPage(SessionResultsPresentationGeneratorPage):

    grok.context(GradSessionResultsPresentationGenerator)

class GradSessionResultsPresentationPDFView(SessionResultsPresentationPDFView):

    grok.context(IGradSessionResultsPresentation)
    grok.name('pdf')
    grok.require('waeup.handleReports')

    def _filename(self):
        return 'GradSessionResultsPresentation_rno%s_%s.pdf' % (
            self.context.__name__,
            self.context.creation_dt_string)